# demoted packages from groovy to jammy
app-install-data
at
calendar
ccache
cpp-10
cpp-10-aarch64-linux-gnu
cpp-10-arm-linux-gnueabihf
cpp-10-doc
cpp-10-powerpc64le-linux-gnu
cpp-10-s390x-linux-gnu
eog-dev
firefox-locale-af
firefox-locale-an
firefox-locale-ar
firefox-locale-as
firefox-locale-ast
firefox-locale-az
firefox-locale-be
firefox-locale-bg
firefox-locale-bn
firefox-locale-br
firefox-locale-bs
firefox-locale-ca
firefox-locale-cak
firefox-locale-cs
firefox-locale-csb
firefox-locale-cy
firefox-locale-da
firefox-locale-de
firefox-locale-el
firefox-locale-en
firefox-locale-eo
firefox-locale-es
firefox-locale-et
firefox-locale-eu
firefox-locale-fa
firefox-locale-fi
firefox-locale-fr
firefox-locale-fy
firefox-locale-ga
firefox-locale-gd
firefox-locale-gl
firefox-locale-gn
firefox-locale-gu
firefox-locale-he
firefox-locale-hi
firefox-locale-hr
firefox-locale-hsb
firefox-locale-hu
firefox-locale-hy
firefox-locale-ia
firefox-locale-id
firefox-locale-is
firefox-locale-it
firefox-locale-ja
firefox-locale-ka
firefox-locale-kab
firefox-locale-kk
firefox-locale-km
firefox-locale-kn
firefox-locale-ko
firefox-locale-ku
firefox-locale-lg
firefox-locale-lt
firefox-locale-lv
firefox-locale-mai
firefox-locale-mk
firefox-locale-ml
firefox-locale-mn
firefox-locale-mr
firefox-locale-ms
firefox-locale-my
firefox-locale-nb
firefox-locale-ne
firefox-locale-nl
firefox-locale-nn
firefox-locale-nso
firefox-locale-oc
firefox-locale-or
firefox-locale-pa
firefox-locale-pl
firefox-locale-pt
firefox-locale-ro
firefox-locale-ru
firefox-locale-si
firefox-locale-sk
firefox-locale-sl
firefox-locale-sq
firefox-locale-sr
firefox-locale-sv
firefox-locale-sw
firefox-locale-ta
firefox-locale-te
firefox-locale-th
firefox-locale-tr
firefox-locale-uk
firefox-locale-ur
firefox-locale-uz
firefox-locale-vi
firefox-locale-xh
firefox-locale-zh-hans
firefox-locale-zh-hant
firefox-locale-zu
fonts-material-design-icons-iconfont
g++-10
g++-10-aarch64-linux-gnu
g++-10-arm-linux-gnueabihf
g++-10-multilib
g++-10-powerpc64le-linux-gnu
g++-10-s390x-linux-gnu
gcc-10
gcc-10-aarch64-linux-gnu
gcc-10-aarch64-linux-gnu-base
gcc-10-arm-linux-gnueabihf
gcc-10-arm-linux-gnueabihf-base
gcc-10-base
gcc-10-cross-base
gcc-10-doc
gcc-10-multilib
gcc-10-powerpc64le-linux-gnu
gcc-10-powerpc64le-linux-gnu-base
gcc-10-s390x-linux-gnu
gcc-10-s390x-linux-gnu-base
gfortran-10
gfortran-10-doc
gfortran-10-multilib
gir1.2-amtk-5
gir1.2-appindicator3-0.1
gir1.2-handy-0.0
gnome-screenshot
golang-1.13-doc
golang-1.13-go
golang-1.13-src
handy-0.0-examples
heimdal-dev
heimdal-docs
heimdal-multidev
initramfs-tools-devices
jasper
leveldb-doc
lib32gcc-10-dev
lib32gfortran-10-dev
lib32go-10-dev
lib32go16
lib32stdc++-10-dev
lib64gcc-10-dev
lib64gfortran-10-dev
lib64go-10-dev
lib64go16
lib64stdc++-10-dev
libamd2
libamtk-5-0
libamtk-5-common
libamtk-5-dev
libappindicator-doc
libasn1-8-heimdal
libasync-mergepoint-perl
libbtf1
libcamd2
libcapnp-dev
libccolamd2
libcholmod3
libcmis-0.5-5v5
libcmis-dev
libconfig-general-perl
libcuda1-340
libcxsparse3
libextutils-pkgconfig-perl
libfuse-dev
libfuse2
libfuture-perl
libgcc-10-dev
libgcc-10-dev-arm64-cross
libgcc-10-dev-armhf-cross
libgcc-10-dev-ppc64el-cross
libgcc-10-dev-s390x-cross
libgetopt-long-descriptive-perl
libgfortran-10-dev
libgl1-mesa-glx
libgo-10-dev
libgo16
libgssapi3-heimdal
libhandy-0.0-0
libhcrypto4-heimdal
libhdb9-heimdal
libheimbase1-heimdal
libheimntlm0-heimdal
libhx509-5-heimdal
libio-async-loop-epoll-perl
libio-async-perl
libkadm5clnt7-heimdal
libkadm5srv8-heimdal
libkafs0-heimdal
libkdc2-heimdal
libklu1
libkrb5-26-heimdal
libldl2
libleveldb-dev
libleveldb1d
liblinux-epoll-perl
libllvm11
liblpsolve55-dev
liblua5.2-0
liblua5.2-dev
libmdds-dev
libmdds-doc
libmetis-dev
libmetis-doc
libmetis5
libmetrics-any-perl
libmongoose2
libmozjs-78-0
libmozjs-78-dev
libneon27-gnutls
libneon27-gnutls-dev
libnss-extrausers
libnumber-range-perl
libopenhpi-dev
libopenhpi3
libotp0-heimdal
libparams-validate-perl
librbio2
libroken18-heimdal
libsereal-perl
libsl0-heimdal
libsliplu1
libsort-key-perl
libspqr2
libstdc++-10-dev
libstdc++-10-dev-arm64-cross
libstdc++-10-dev-armhf-cross
libstdc++-10-dev-ppc64el-cross
libstdc++-10-dev-s390x-cross
libstdc++-10-doc
libstdc++-10-pic
libstruct-dumb-perl
libsuitesparse-dev
libsuitesparse-doc
libtest-fatal-perl
libtest-metrics-any-perl
libtest-refcount-perl
libucpp-dev
libumfpack5
libwind0-heimdal
libx32gcc-10-dev
libx32gfortran-10-dev
libx32go-10-dev
libx32go16
libx32stdc++-10-dev
libxml-writer-perl
libxmlrpc-epi-dev
libxmlrpc-epi0
libxmlrpc-epi0-dbg
locales-all
ltrace
lua5.2-doc
ncal
node-jquery
nvidia-opencl-icd-340
ocl-icd-dev
ocl-icd-libopencl1
ocl-icd-opencl-dev
oem-config-debconf
opencl-c-headers
opencl-clhpp-headers
opencl-clhpp-headers-doc
openhpi
openhpi-clients
openhpi-plugin-dynamic-simulator
openhpi-plugin-ilo2-ribcl
openhpi-plugin-ipmi
openhpi-plugin-ipmidirect
openhpi-plugin-oa-soap
openhpi-plugin-ov-rest
openhpi-plugin-simulator
openhpi-plugin-slave
openhpi-plugin-snmp-bc
openhpi-plugin-sysfs
openhpi-plugin-test-agent
openhpi-plugin-watchdog
openhpid
php-xmlrpc
popularity-contest
probert-network
python-parted-doc
python3-bsddb3
python3-ecdsa
python3-libxml2
python3-linecache2
python3-parted
python3-pil.imagetk
python3-pyxs
python3-requests-unixsocket
qemu
ruby-minitest
ruby-power-assert
ruby-test-unit
sharutils
sharutils-doc
tasksel
tasksel-data
tegaki-zinnia-japanese
tgt
ubuntu-fan
ubuntu-wallpapers-groovy
ubuntu-wallpapers-groovy-raspi
ucpp
vino
wslu
xul-ext-ubufox
